# Games::Checkers, Copyright (C) 1996-2004 Mikhael Goikhman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

package Games::Checkers::MoveConstants;

use Games::Checkers::Move;
use Games::Checkers::DeclareConstant NO_MOVE => Games::Checkers::Move::NoMove;

1;
